clear all; close all; clc;
%% CAM SHAPE DETERMINATION
%Code for determining the cam shape based on a given torque profile using
%an ordinairy coil spring. Based on the work of Tidwell (Wrapping cam
%mechanisms). The code incrementally solves for the cam shape given the a
%specified input (pulley) and output (Cam) torque. The input and output
%torque needs to specified using the functions Tcam and Tpulley. Note that
%for not all profiles a solution can be found.
% Created by B.J. van der Burgh on 5-9-2024

%Initialise variables
k=10; %Spring stiffness
x0=0.1; %Initial pretension length of the spring
t=0.01; %Belt thickness
r=0.5; %Radius of pulley
rp=r+t/2; %Pulley pitch radius 
c=10; %Centerline distance between the cam and pulley


Thetamax=270/180*pi; %Max rotation angle of cam
dTheta=0.1/180*pi; %Angle increment factor
n=Thetamax/dTheta+1; %Number of steps for which to solve the cam shape
eps=10^-5; %error criteria for iterative solver

%Initialise arrays
theta=linspace(0,Thetamax,n); %Rotation angle of the cam
psi=zeros(1,n); %Rotation angle of the pulley
Tc=zeros(1,n); %Torque around the cam
Tp=zeros(1,n); %Torque around the pulley
dTc=zeros(1,n); %Derivative of Tc with respect to theta
dTp=zeros(1,n); %Derivative of Tp with respect to theta
h=zeros(1,n); %Moment arm of the cam
p=zeros(1,n); %Cam shape

%Initialisation step (theta and psi are zero)
[Tc(1),dTc(1)]=Tcam(0); %Torque around cam
[Tp(1),dTp(1)]=Tpulley(0,k,rp,x0,Tc(1)); %Torque around pulley
h(1)=c_h(Tc(1),Tp(1),rp); %Moment arm of the cam

for e=2:n %Incrementally solve for the moment arm
    [Tc(e),dTc(e)]=Tcam(theta(e));
    
    %As psi depends on h an initial guess is made of psi based on h in the
    %previous time step.
    psi_est=1/rp*h(e-1)*dTheta+psi(e-1);
    [Tp(e),dTp(e)]=Tpulley(psi_est,k,rp,x0,Tc(e));
    h(e)=c_h(Tc(e),Tp(e),rp);
    %Recalculate psi using new h based on trapezoidal integration rule
    psi(e)=dTheta/(2*rp)*(h(e)+h(e-1))+psi(e-1);
    %Update psi, psi_est and h till psi_est converges to psi
    iter=0;
    while (abs(psi(e)-psi_est))>eps && iter<100
        iter=iter+1;
        psi_est=psi(e);
        [Tp(e),dTp(e)]=Tpulley(psi_est,k,rp,x0,Tc(e));
        h(e)=c_h(Tc(e),Tp(e),rp);
        psi(e)=dTheta/(2*rp)*(h(e)+h(e-1))+psi(e-1);
        if iter==99 %Maximum number of iterations
            disp('no convergence')
        end
    end
end

%Determine the shape of the cam expressed in complex polar form
for i=1:n
    p(i)=shape(theta(i),c,rp,t,h(i),Tp(i),dTp(i),Tc(i),dTc(i));
end

%Plot torque-angle curve
figure
plot(theta*180/pi,Tc)
xlabel('cam angle [deg]')
ylabel('Torque [Nm]')
title('torque-angle characteristics of the cam')

%Plot cam shape
figure
plot(real(p),imag(p))
hold on
plot(0,0,'ko','MarkerSize',10,'LineWidth',2) %Centre of rotation of the cam
axis equal
xlabel('x [mm]')
ylabel('y [mm]')
title('cam shape')

function [Tc,dTc]=Tcam(theta)
    %Calculate the torque around the cam and its derivative for a given
    %angle

    %Circular cam (linear profile). 
    %For a circular cam it must hold that theta0 (pretension)
    %equals x0/sqrt(kT/k). Where kT can be thought of as the torsional
    %stiffness of the cam (in the case of a linear profile)
    Tc=0.5*(theta+0.1/sqrt(0.5/10));
    dTc=0.5;

    Tc=0.5*(theta+0.1/sqrt(0.5/10))+0.5*theta^2-0.075*theta^3;
    dTc=0.5+theta;
end

function [Tp,dTp]=Tpulley(psi,k,rp,x0,Tc)
    %Calculate the torque around the pulley and its derivative for a given
    %angle psi. Here the torque is defined as Tp=rp*F. With F=k*(rp*psi+x0)
    Tp=k*rp*(rp*psi+x0);
    dTp=k*rp^2*Tc/Tp;
end

function [h]=c_h(Tc,Tp,rp)
    %Calculate the moment arm of the force F at the cam
    h=rp*Tc/Tp;
end

function [p]=shape(theta,c,rp,t,h,Tp,dTp,Tc,dTc)
    %Calculate the shape of the cam
    dh=rp*(Tp*dTc-Tc*dTp)/Tp^2; %Derivative of h with respect to theta
    phi=acos((h-rp)/c); %Angle between the centerline (c) and the moment arm (h)
    dphi=-1/sqrt(c^2-(h-rp)^2)*dh; %Derivative of phi with respect to theta
    %Shape of the cam in complex polar form
    p=c*exp(1i*theta)+(rp-t/2)*exp(1i*(theta+phi))+c*sin(phi)/(1+dphi)*exp(1i*(theta+phi+pi/2));
end